#pragma rtGlobals = 1		// modern global method

#include <Strings as Lists>

// PPT Dose Response
// part of the PPT, the "Patcher's Power Tools"
//  17.03.97 Francisco Mendez

// 18.03.97 new control panel: Dose-Response
// 20.03.97 display parameter will be calculated if not specified
// 03.07.97 fixed parameters will not be estimated
// 03.07.97 new button to append a legend with fit results to the graph
// 23.10.97 macro stores PPT-globals in folder "PPTGlobals", Igor Globals are not created
// 27.10.97 changed all procedures to functions

// ****************************************************

Menu "Macros"
	"Dose Response Panel", PPTDoseResponse ()
End

// ****************************************************

Function PPTDoseResGlobals ()
	String oldDataFolder = GetDataFolder (1)
	if (! DataFolderExists ("root:PPTGlobals"))
		NewDataFolder /O root:PPTGlobals
	endif

	SetDataFolder root:PPTGlobals

	Variable /G	pptYMax1		// first maximal value
	Variable /G	pptYMax2		// second maximal value
	Variable /G	pptKD1			// first KD 
	Variable /G	pptKD2			// second KD value
	Variable /G	pptHill1		// first Hill coefficient
	Variable /G	pptHill2		// second Hill coefficient

	Variable /G	pptDRMin		// min. x-Value
	Variable /G	pptDRMax		// max. x-Value for display
	Variable /G pptDRNum		// number of points

	SetDataFolder $oldDataFolder
End Function

// ****************************************************

Function PPTDoseResponse () : Panel
	Silent 1; PauseUpdate

	Variable	panelX
	variable	panelY
	Variable	panelW = 352
	Variable	panelH = 170
	Variable	V_right, V_top

	if (strlen (WinName (0, 1)) <= 0)
		Abort "Please create a graph before doing any dose-response fit!"
	endif

	GetWindow kwTopWin, wsize	// place control panel to the right of the top graph
	panelX = V_right
	panelY = V_top

	PPTDoseResGlobals ()			// create global variables
	NewPanel /W=(panelX, panelY, panelX + panelW, panelY + panelH) as "PPT Dose-Response"
	Execute "ModifyPanel cbRGB = (61166, 61166, 61166)"		// grey background

	// Y Max 1
	CheckBox cYMax1	pos = {4, 2}, size = {18, 19}, title = ""
	CheckBox cYMax1	value = 0
	CheckBox cYMax1	help = {"If cecked, maximal value will not be fitted."}
	DrawText 30, 19, "Max"
	SetVariable vYMax1	pos = {80, 4}, size = {80, 14}, title = " "
	SetVariable vYMax1	format ="%g", limits = {-INF, INF, 0}
	SetVariable vYMax1	value = root:PPTGlobals:pptYMax1
	SetVariable vYMax1	help = {"Sets and shows maximal value."}

	// KD 1
	CheckBox cKD1 pos = {4, 21}, size = {18, 19}, title = ""
	CheckBox cKD1 value = 0
	CheckBox cKD1 help = {"If cecked, KD will not be fitted."}
	DrawText 30, 38, "KD"
	SetVariable vKD1	pos = {80, 23}, size = {80, 14}, title = " "
	SetVariable vKD1	format ="%g", limits = {-INF, INF, 0}
	SetVariable vKD1	value = root:PPTGlobals:pptKD1
	SetVariable vKD1	help = {"Sets and shows KD Value."}

	// Hill 1
	CheckBox cHill1	pos = {4, 40}, size = {18, 19}, title = ""
	CheckBox cHill1	value = 0
	CheckBox cHill1	help = {"If cecked, Hill coefficient will not be fitted."}
	DrawText 30, 56, "Hill"
	SetVariable vHill1	pos = {80, 42}, size = {80, 14}, title = " "
	SetVariable vHill1	format ="%g", limits = {-INF, INF, 0}
	SetVariable vHill1	value = root:PPTGlobals:pptHill1
	SetVariable vHill1	help = {"Sets and shows Hill coefficient value."}

	CheckBox	cCursor	pos = {4, 58}, size = {120, 19}, title = "Use Cursors"
	CheckBox	cCursor	value = 0
	CheckBox	cCursor	help = {"If checked, only cursor range will be fitted."}

	// Y Max 2
	CheckBox cYMax2	pos = {188, 4}, size = {18, 19}, title = ""
	CheckBox cYMax2	value = 0
	CheckBox cYMax2	help = {"If cecked, second maximal value will not be fitted."}
	DrawText 212, 19, "Max 2"
	SetVariable vYMax2	pos = {268, 6}, size = {80, 14}, title = " "
	SetVariable vYMax2	format ="%g", limits = {-INF, INF, 0}
	SetVariable vYMax2	value = root:PPTGlobals:pptYMax2
	SetVariable vYMax2	help = {"Sets and shows second maximal value."}

	// KD 2
	CheckBox cKD2		pos = {188, 21}, size = {18, 19}, title = ""
	CheckBox cKD2		value = 0
	CheckBox cKD2		help = {"If cecked, second KD will not be fitted."}
	DrawText 212, 38, "KD 2"
	SetVariable vKD2	pos = {268, 23}, size = {80, 14}, title = " "
	SetVariable vKD2	format ="%g", limits = {-INF, INF, 0}
	SetVariable vKD2	value = root:PPTGlobals:pptKD2
	SetVariable vKD2	help = {"Sets and shows second KD Value."}

	// Hill 2
	CheckBox cHill2	pos = {188, 40}, size = {18, 19}, title = ""
	CheckBox cHill2	value = 0
	CheckBox cHill2	help = {"If cecked, second Hill coefficient will not be fitted."}
	DrawText 212, 56, "Hill 2"
	SetVariable vHill2	pos = {268, 42}, size = {80, 14}, title = " "
	SetVariable vHill2	format ="%g", limits = {-INF, INF, 0}
	SetVariable vHill2	value = root:PPTGlobals:pptHill2
	SetVariable vHill2	help = {"Sets and shows second Hill coefficient value."}

	CheckBox	cDblDR	pos = {188, 58}, size = {120, 19}, title = "Double Dose-R."
	CheckBox	cDblDR	value = 0
	CheckBox	cDblDR	help = {"If checked, double dose-response will be fitted"}

	// PopupMenus with x- and y-Wave
	PopupMenu pYWave	pos = {4, 82}, size = {140, 19}
	PopupMenu pYWave	title = "y-Data"
	PopupMenu pYWave	mode = 1
	PopupMenu	pYWave	value = PPTDRYWave ()
	PopupMenu pYWave	help = {"Selects an y-wave (Effect) for dose-response."}
	PopupMenu pXWave	pos = {4, 104}, size = {140, 19}
	PopupMenu pXWave	title = "x-Data"
	PopupMenu pXWave	mode = 1
	PopupMenu	pXWave	value = PPTDRXWave ()
	PopupMenu pXWave	help = {"Selects an x-wave (Concentration) for dose-response."}

	SetVariable vDRMin		pos = {268, 82}, size = {80, 14}, title = "x-min"
	SetVariable vDRMin		format ="%g", limits = {-INF, INF, 0}
	SetVariable vDRMin		value = root:PPTGlobals:pptDRMin
	SetVariable vDRMin		help = {"Minimal x-value of fitted curve."}
	SetVariable vDRMin		proc = PPTChangeFit

	SetVariable vDRMax		pos = {268, 99}, size = {80, 14}, title = "x-max"
	SetVariable vDRMax		format ="%g", limits = {-INF, INF, 0}
	SetVariable vDRMax		value = root:PPTGlobals:pptDRMax
	SetVariable vDRMax		help = {"Maximal x-value of fitted curve."}
	SetVariable vDRMax		proc = PPTChangeFit

	SetVariable vDRNum		pos = {268, 116}, size = {80, 14}, title = "n"
	SetVariable vDRNum		format ="%d", limits = {0, INF, 100}
	SetVariable vDRNum		value = root:PPTGlobals:pptDRNum
	SetVariable vDRNum		help = {"Number of values of fitted curve."}
	SetVariable vDRNum		proc = PPTChangeFit

	Button		bAppendLegend	pos = {4, 126}, size = {110, 19}, title = "Append Legend"
	Button		bAppendLegend	proc = PPTAppendLegend

	Button		bRemoveLegend	pos = {120, 126}, size = {110, 19}, title = "Remove Legend"
	Button		bRemoveLegend	proc = PPTRemoveLegend

	Button		bDoDoseRes		pos = {4, 146}, size = {110, 19}, title = "Fit Dose-Res."
	Button		bDoDoseRes		proc = PPTDoDoseResponse

	Button		bEstimDoseRes	pos = {120, 146}, size = {110, 19}, title = "Auto Estimate"
	Button		bEstimDoseRes	proc = PPTEstimateDoseResponse

	Button		bDrawDoseRes		pos = {238, 146}, size = {110, 19}, title = "Draw Fit"
	Button		bDrawDoseRes		proc = PPTDrawDoseResponse

	ResumeUpdate
End Window

// ****************************************************

Function /S PPTDRYWave ()
	PopupMenu pXWave mode = 1
	return (WaveList ("*", ";", "WIN:"))
End Function

// ****************************************************

Function /S PPTDRXWave ()
	String		theXWaveList = "_calculated_;"
	String		theWaveList
	String		theYWave, theString
	Variable	num
	Variable	index = 0
	String		S_Value

	ControlInfo pYWave
	theYWave = S_Value
	num = numpnts ($theYWave)
	theWaveList = WaveList ("*", ";", "WIN:")

	do
		// get the next wave name
		theString = GetStrFromList (theWaveList, index, ";")
		if (strlen (theString) == 0)
			break
		endif

		if (numpnts ($theString) == num)
			theXWaveList += theString
			theXWaveList += ";"
		endif

		index += 1
	while (1)

	return (theXWaveList)
End Function


// ****************************************************

Function PPTAppendLegend (ctrlName):  ButtonControl
	String	ctrlName
	
	String	theLegend

	NVAR	yMax1	= root:PPTGlobals:pptYMax1
	NVAR	yMax2	= root:PPTGlobals:pptYMax2
	NVAR	kD1	= root:PPTGlobals:pptKD1
	NVAR	kD2	= root:PPTGlobals:pptKD2
	NVAR	hill1	= root:PPTGlobals:pptHill1
	NVAR	hill2 	= root:PPTGlobals:pptHill2

	if (PPTDoDoubleDoseResponse ())
		sprintf theLegend, "Double Dose-Response\rMax1 = %g, Max2 = %g\rKD1 = %g, KD2 = %g\rHill1 = %g, Hill2 = %g", yMax1, yMax2, kD1, kD2, hill1, hill2
	else
		sprintf theLegend, "Single Dose-Response\rMax = %g\rKD = %g\rHill = %g", yMax1, kD1, hill1
	endif

	TextBox /C/N=DR_Legend theLegend
End Function

// ****************************************************

Function PPTRemoveLegend (ctrlName):  ButtonControl
	String	ctrlName

	Textbox/K/N=DR_Legend
End Function

// ****************************************************

Function PPTEstimateDoseResponse (ctrlName):  ButtonControl
	String		ctrlName

	String		yWaveName, xWaveName
	Variable	yKD, yMin, yMax, kDloc
	Variable	yMax1, yMax2, kD1, kD2, hill1, hill2
	Variable	doDbl, increasing
	Variable	V_Value, V_npnts, V_min, V_max
	String		S_Value

	ControlInfo cDblDR
	doDbl = V_Value

	ControlInfo pXWave
	xWaveName = S_Value
	WAVE xWave = $xWaveName
	if ((strlen (xWaveName) <= 0) %| (WaveExists (xWave) == 0))
		Abort "Please select the x-data from the popup!"
	endif

	ControlInfo pYWave
	yWaveName = S_Value
	WAVE yWave = $yWaveName
	if ((strlen (yWaveName) <= 0) %| (WaveExists (yWave) == 0))
		Abort "Please select the y-data from the popup!"
	endif

	WaveStats /Q yWave
	increasing = (yWave [V_npnts] > yWave [0])
	yMin = V_min
	yMax = V_max
	yKD = yMin + (yMax - yMin) / 2

	if (doDbl)
		yMax1 = yKD
		yMax2 = yKD
		kD1 = xWave [BinarySearch (yWave, yKD - (yKD - yMin) / 2)]
		kD2 = xWave [BinarySearch (yWave, yMax - (yMax - yKD) / 2)]
		if (increasing)
			hill1 = -1
			hill2 = -1
		else
			hill1 = 1
			hill2 = 1
		endif
	else
		yMax1 = yMax
		kD1 = xWave [BinarySearch (yWave, yKD)]
		if (increasing)
			hill1 = 1
		else
			hill1 = -1
		endif
	endif

	// change parameter only if it is not fixed
	NVAR	pptMax1	= root:PPTGlobals:pptYMax1
	NVAR	pptMax2	= root:PPTGlobals:pptYMax2
	NVAR	pptKD1		= root:PPTGlobals:pptKD1
	NVAR	pptKD2		= root:PPTGlobals:pptKD2
	NVAR	pptHill1	= root:PPTGlobals:pptHill1
	NVAR	pptHill2 	= root:PPTGlobals:pptHill2

	if (! PPTGetCheckBox ("cYMax1"))
		pptMax1 = yMax1
	endif
	if (! PPTGetCheckBox ("cYMax2") %& doDbl)
		pptMax2 = yMax2
	endif
	if (! PPTGetCheckBox ("cKD1"))
		pptKD1 = kD1
	endif
	if (! PPTGetCheckBox ("cKD2") %& doDbl)
		KD2 = kD2
	endif
	if (! PPTGetCheckBox ("cHill1"))
		pptHill1 = hill1
	endif
	if (! PPTGetCheckBox ("cHill2") %& doDbl)
		pptHill2 = hill2
	endif
End Proc

// ****************************************************

Function PPTDoDoseResponse (ctrlName):  ButtonControl
	String		ctrlName

	Variable	doDbl, useCursors
	String		yWaveName, xWaveName, coeffsWaveName
	String		theCmd
	Variable	num
	Variable	V_Value
	String		S_Value

	// Global lookups
	NVAR	yMax1	= root:PPTGlobals:pptYMax1
	NVAR	yMax2	= root:PPTGlobals:pptYMax2
	NVAR	kD1	= root:PPTGlobals:pptKD1
	NVAR	kD2	= root:PPTGlobals:pptKD2
	NVAR	hill1	= root:PPTGlobals:pptHill1
	NVAR	hill2 	= root:PPTGlobals:pptHill2

	ControlInfo cDblDR
	doDbl = V_Value
	ControlInfo cCursor
	useCursors = V_Value

	ControlInfo pYWave
	yWaveName = S_Value
	WAVE yWave = $yWaveName
	if ((strlen (yWaveName) <= 0) %| (WaveExists (yWave) == 0))
		Abort "Please select the y-data from the popup!"
	endif
	num = numpnts (yWave)
	if (doDbl %& (num < 6))
		Abort "Double Dose Response fits 6 parameters so it requires at least 6 values!"
	endif
	if (num < 3)
		Abort "Dose Response fits 3 parameters so it requires at least 3 values!"
	endif
	if (useCursors)
		if ((cmpstr (yWaveName, csrWave (A))) %| (cmpstr (yWaveName, csrWave (B))))
			Abort "If the cursor range has to be fitted, you must set both cursors on the specified y-wave!"
		endif
	endif

	ControlInfo pXWave
	xWaveName = S_Value
	WAVE xWave = $xWaveName
	if ((V_Value < 2) %| (WaveExists (xWave) == 0))
		Abort "Please select the x-data from the popup!"
	endif

	// Holding parameters
	theCmd = "FuncFit /Q/H=\""
	theCmd += PPTCheckToBoolStr ("cYMax1")
	theCmd += PPTCheckToBoolStr ("cKD1")
	theCmd += PPTCheckToBoolStr ("cHill1")
	if (doDbl)
		theCmd += PPTCheckToBoolStr ("cYMax2")
		theCmd += PPTCheckToBoolStr ("cKD2")
		theCmd += PPTCheckToBoolStr ("cHill2")
	endif
	theCmd += "\" "

	// FitFunction
	if (doDbl)
		theCmd += "PPTDblDoseResponseFunc "
	else
		theCmd += "PPTDoseResponseFunc "
	endif

	// Coefficients
	coeffsWaveName = yWaveName + "_coeffs"
	if (doDbl)
		Make /D/O/N=6 $coeffsWaveName
	else
		Make /D/O/N=3 $coeffsWaveName
	endif
	WAVE coeffswave = $coeffsWaveName
	coeffsWave [0] = yMax1
	coeffsWave [1] = kD1
	coeffsWave [2] = hill1
	if (doDbl)
		coeffsWave [3] = yMax2
		coeffsWave [4] = kD2
		coeffsWave [5] = hill2
	endif
	theCmd += coeffsWaveName + " "

	// y-Wave
	theCmd += yWaveName
	if (useCursors)
		theCmd += "(xcsr(A),xcsr(B))"
	endif
	theCmd += " "

	// x-Wave
	theCmd += ("/X=" + xWaveName + " /D")

	Execute theCmd

	PPTSetFitParams (coeffsWave, doDbl)

	Execute "Remove fit_" + yWaveName
	PPTDrawDoseResponse (ctrlName)
End Proc

// ****************************************************

Function PPTSetFitParams (theWave, doDbl)
	Wave		theWave
	Variable	doDbl

	NVAR	yMax1	= root:PPTGlobals:pptYMax1
	NVAR	yMax2	= root:PPTGlobals:pptYMax2
	NVAR	kD1		= root:PPTGlobals:pptKD1
	NVAR	kD2		= root:PPTGlobals:pptKD2
	NVAR	hill1	= root:PPTGlobals:pptHill1
	NVAR	hill2 	= root:PPTGlobals:pptHill2

	yMax1	= theWave [0]
	kD1	= theWave [1]
	hill1	= theWave [2]
	if (doDbl)
		yMax2	= theWave [3]
		kD2	= theWave [4]
		hill2	= theWave [5]
	endif
End Function

// ****************************************************
// Check if wave is in top graph, returns 0 if not, 1 else
// ****************************************************

Function PPTWaveInTopGraph (waveName)
	String waveName

	String theWaveList = WaveList ("*", ";", "WIN:")
	if (strsearch (LowerStr (theWaveList), LowerStr (waveName), 0) >= 0)
		return 1
	else
		return 0
	endif
End Function

// ****************************************************
// Redraw the fitted curve based upon new settings, returns 0, if fitted graph doesn't exist
// ****************************************************

Function PPTChangeFit (ctrlName, varNum, varStr, varName) : SetVariableControl
	String		ctrlName
	Variable	varNum
	String		varStr
	String		varName

	if (cmpstr (ctrlName [0,2], "vDR") != 0)
		return 0		// wrong call
	endif

	String waveName, S_Value
	ControlInfo pYWave
	waveName = S_Value + "_fit_y"
	if (PPTWaveInTopGraph (waveName))
		PPTDrawDoseResponse (ctrlName)
		return 1
	else
		return 0
	endif
End Function

// ****************************************************

Function PPTDrawDoseResponse (ctrlName):  ButtonControl
	String		ctrlName
	
	Variable	doDbl
	String		yWaveName, xWaveName, theWaveList
	String		fitYWaveName, fitXWaveName
	Variable	V_Value, V_min, V_max
	String		S_Value

	NVAR	yMax1	= root:PPTGlobals:pptYMax1
	NVAR	yMax2	= root:PPTGlobals:pptYMax2
	NVAR	kD1		= root:PPTGlobals:pptKD1
	NVAR	kD2		= root:PPTGlobals:pptKD2
	NVAR	hill1	= root:PPTGlobals:pptHill1
	NVAR	hill2 	= root:PPTGlobals:pptHill2
	NVAR	drNum	= root:PPTGlobals:pptDRNum
	NVAR	drMin	= root:PPTGlobals:pptDRMin
	NVAR	drMax	= root:PPTGlobals:pptDRMax

	ControlInfo pYWave
	yWaveName = S_Value
	WAVE yWave = $yWaveName
	if (((strlen (yWaveName) <= 0)) %| (WaveExists (yWave) == 0))
		Abort "You have to specify the y-data to determine the base name of the fit!"
	endif

	if (drNum < 100)						// do not use too few values in fit
		drNum = 400
	endif
	if ((drMin == drMax) %| (drMin <= 0.0) %| (drMax <= 0.0))
	 	ControlInfo pXWave
		xWaveName = S_Value
		WAVE xWave = $xWaveName
	 	if ((V_Value < 2) %| (WaveExists (xWave) == 0))
			Abort "x-min and x-max must differ! Otherwise specify an x-wave to auto-estimate those values!"
		else
			WaveStats /Q xWave
			drMin = V_min
			drMax = V_max
		endif
	endif

	ControlInfo cDblDR
	doDbl = V_Value

	yWaveName += "_fit"
	WAVE yWave = $yWaveName 
	fitYWaveName = yWaveName + "_y"
	fitXWaveName = yWaveName + "_x"

	if (PPTWaveInTopGraph (fitYWaveName))
		RemoveFromGraph $fitYWaveName
	endif

	PPTDoMakeLogXWave (yWaveName, drNum, drMin, drMax, 1)

	WAVE fitYWave = $fitYWaveName
	WAVE fitXWave = $fitXWaveName
	if (doDbl)
		fitYWave = yMax1 / (1 + (fitXWave / kD1) ^ hill1) + yMax2 / (1+(fitXWave / kD2) ^ hill2)
	else
		fitYWave = yMax1 * (1 / (1 + (kD1 / fitXWave) ^ hill1))
	endif
	AppendToGraph fitYWave vs fitXWave
End Proc

// ****************************************************

Function /S PPTCheckToBoolStr (ctrlName)
	String		ctrlName
	Variable	V_Value

	ControlInfo $ctrlName
	if (V_Value)
		return ("1")
	else
		return ("0")
	endif
End Function

// ****************************************************

Function PPTGetCheckBox (ctrlName)
	String		ctrlName
	Variable	V_Value

	ControlInfo $ctrlName
	return (V_Value)
End Function

// ****************************************************

Function PPTDoseResponseFunc (theWave, xVal)		
	Wave		theWave
	Variable	xVal

	// theWave [0]	YMax
	// theWave [1]	Kd
	// theWave [2]	Hill

	return (theWave [0] * (1 / (1 + (theWave [1] / xVal) ^ theWave [2])))
End Function

// ****************************************************

Function PPTDblDoseResponseFunc (theWave, xVal)		
	Wave		theWave
	Variable	xVal

	// theWave [0]	YMax1
	// theWave [1]	KD1
	// theWave [2]	Hill1
	// theWave [3]	YMax2
	// theWave [4]	Kd2
	// theWave [5]	Hill2

	return (theWave [0] / (1 + (xVal / theWave [1]) ^ theWave [2]) + theWave [3] / (1+(xVal / theWave[4]) ^ theWave [5]))
End // PPTDblDoseResponseFunc

// ****************************************************

Function PPTDoDoubleDoseResponse ()
	Variable	V_Value

	ControlInfo cDblDR
	return (V_Value)
End Function